% function [Ad,Bd,Cd,Dd,As,Ks,Cs,Lambda]=ort_dec(y,u,ns,nd,ks,kd,BD,T,delay,type,Aest,kbd);
%
%
% (Ad,Bd,Cd,Dd) is a state space realization of the deterministic subsystem
% (As,Ks,Cs,I) is an innovation representation of the stochastic component
%  Lambda is the variance of the innovation
%
%  y : output data as row vector y = [y(1), y(2),......,y(Ndat)]
%  u : input data as row vector u = [u(1), u(2),......,u(Ndat)]
%  ks, kd dimension of spaces in deterministic and stochastic identification
%  ns, nd are the choosen orders
%
%  BD : determination of the matrices B and D
%      BD = 'mpm' : minimum prediction error
%      BD = 'msm' : minimum simulation error
%      BD = 'msm_block' : minimum simulation error on the columns of the 
%                         estimated deterministic component
%      BD = 'proj' : projection approach (e.g.  MOESP)
%      BD = 'proj_ls' : projection approach (e.g.  MOESP)
%                       with orthonormalization
%      BD ='proj_ls_white' : projection approach (e.g. MOESP) with 
%          "pre-whitening" of the residuals ...
%      BD ='proj_ls_white_d' projection approach (e.g. MOESP) with 
%          "pre-whitening"  ... use only det obs. matrix 
%           
%  default : BD ='proj_ls'
%
%  type : identification of the deterministic part ;
%      type = 'ort' : orthogonal projection
%      type = 'obl' : oblique projection
%      type = 'cva' : Canonical Variate Analysis
%  default type = 'ort'
% 
% Aest : choice of identification method for A
%        Aest = 'TLS' total least square solution
%        Aest = 'SI' shift invariance
%        Aest = 'ls' state sequence approach
% default Aest ='TLS'

function [Ad,Bd,Cd,Dd,As,Ks,Cs,Lambda]=ort_dec_7_00(y,u,ns,nd,ks,kd,BD,T,delay,type,Aest,kbd);

if ((nargin > 12) | (nargin < 2)) 
   disp('Wrong number of input arguments')
   return
end;


if ((nargin < 3) | isempty(ns))
   ns=[1:10];
end;

if ((nargin < 4) | isempty(nd))
   nd = [1:10];
end;

if ((nargin < 5) | isempty(ks))
   ks = ceil(1.2*max(ns)+3); % as in the N4SID file of matlab
end;

if ((nargin < 6) | isempty(kd))
   kp= ceil(1.2*max(ns)+3); % as in the N4SID file of matlab
   kf = kp;
else
   kp = kd(1);
   kf = kd(2);
end;
if (nargin < 7  | isempty(BD))
   BD = 'proj_ls';
end;

if (nargin < 8 | isempty(T))
   T = 100;
end;

if (nargin < 9  | isempty(delay))
   delay = 0;
end;

if (nargin < 10  | isempty(type))
   type = 'ort';
end;

if (nargin < 11  | isempty(Aest))
   Aest = 'TLS';
end;

if (nargin < 12  | isempty(kbd))
   kbd = kf;
end;

[m,Nu]=size(u);
[p,Ny]=size(y);


N = min(Nu,Ny)-(kf+kp)+1;

%%% Hankel matrices:


U = Hankel(u,kf+kp,N);
Y = Hankel(y,kf+kp,N);
Up = U(1:kp*m,:);
Uf = U(kp*m+1:(kf+kp)*m,:);
Yp = Y(1:kp*p,:);
Yf = Y(kp*p+1:(kf+kp)*p,:);

H = [Uf; Up ; Y];

[Q,R] =qr(H',0); % economy size qr decomposition 
Q=Q(:,1:(kf+kp)*(m+p))';
R=triu(R(1:(kf+kp)*(m+p),1:(kf+kp)*(m+p)))';

if (strcmp(BD,'proj_ls')|strcmp(BD,'proj')|strcmp(BD,'proj_ls_white')|strcmp(BD,'proj_ls_white_d')|strcmp(BD,'proj_ls_white_new'))
   
   [UUU SSS VVV]=svd(U(1:kbd*m,:));
   rr = sum(diag(SSS)>eps);
   UUU = UUU(:,1:rr);
   SSS = SSS(1:rr,1:rr);
   VVV = VVV(:,1:rr);
   SSS = diag(SSS);
   [rv cv]=size(VVV);
   Phi = Y(1:kbd*p,:)*VVV;
end;

if strcmp(BD,'proj')
   Phi = Phi*pinv(diag(SSS))*UUU';
end;


Qys = Q((kf+kp)*m+1:(kf+kp)*(m+p),:);
Qyd = Q(1:(kf+kp)*m,:);

Yd = R((kf+kp)*m+1:(kf+kp)*(m+p),1:(kf+kp)*m)*Qyd;
Ys = R((kf+kp)*m+1:(kf+kp)*(m+p),(kf+kp)*m+1:(kf+kp)*(m+p))*Qys; %Y -Yd;

% compute deterministic matrices Ad,Cd

if strcmp(Aest,'ls')
   if strcmp(type,'ort'),
    [U S V] = svd(R((kf+kp)*m+kp*p+1:(kf+kp)*(p+m)-p,kf*m+1:(kf+kp)*m)); % orthogonal projection
   elseif strcmp(type,'cva'),
    Yup = R((kf+kp)*m+kp*p+1:(kf+kp)*(m+p)-p,kf*m+1:(kf+kp)*(m+p)-p);
    
    Lup = chol(Yup*Yup');
    [U S V] = svd(pinv(Lup')*R((kf+kp)*m+kp*p+1:(kf+kp)*(p+m)-p,kf*m+1:(kf+kp)*m)); % cva
    %U = Lup'*U;
   elseif strcmp(type,'obl')
    [U S V] = svd(R((kf+kp)*m+kp*p+1:(kf+kp)*(p+m)-p,kf*m+1:(kf+kp)*m)...
      *pinv(R(kf*m+1:(kf+kp)*m,kf*m+1:(kf+kp)*m))*R(kf*m+1:(kf+kp)*m,1:(kf+kp)*m));
    % oblique projection
   else
   disp('Wrong choice of method for estimation of the observability matrix')
   return
 end;
else
 if strcmp(type,'ort'),
   [U S V] = svd(R((kf+kp)*m+kp*p+1:(kf+kp)*(p+m),kf*m+1:(kf+kp)*m)); % orthogonal projection
elseif strcmp(type,'cva'),
   
   Yup = R((kf+kp)*m+kp*p+1:(kf+kp)*(m+p),kf*m+1:(kf+kp)*(m+p));
   Lup = chol(Yup*Yup');
   [U S V] = svd(pinv(Lup')*R((kf+kp)*m+kp*p+1:(kf+kp)*(p+m),kf*m+1:(kf+kp)*m)); % cva
   U = Lup'*U;
elseif strcmp(type,'obl')
   [U S V] = svd(R((kf+kp)*m+kp*p+1:(kf+kp)*(p+m),kf*m+1:(kf+kp)*m)...
      *pinv(R(kf*m+1:(kf+kp)*m,kf*m+1:(kf+kp)*m))*R(kf*m+1:(kf+kp)*m,1:(kf+kp)*m));
   % oblique projection
else
   disp('Wrong choice of method for estimation of the observability matrix')
   return
 end;
end;

if strcmp(type,'cva'),
   if strcmp(Aest,'ls')
      Ystemp = pinv(Lup')*Ys(kp*p+1:(kf+kp)*p-p,:);
      syh = norm(Ystemp(p*(kf-1)+1:p*kf-p,:)*Ystemp(p*(kf-1)+1:p*kf-p,:)','fro');
   else  
      Ystemp = pinv(Lup')*Ys(kp*p+1:(kf+kp)*p,:);
      syh = norm(Ystemp(p*(kf-1)+1:p*kf,:)*Ystemp(p*(kf-1)+1:p*kf,:)','fro');
   end;
else
   syh = norm(Ys((kf+kp)*p+1-p:(kf+kp)*p,:)*Ys((kf+kp)*p+1-p:(kf+kp)*p,:)','fro');
end;

if min(nd)<max(nd),     % if def_order
   
   
      figure
      dS=diag(S);
      testo=(diag(S));
      ndef=max(max(find(testo>(max(testo)+min(testo))/2)));
      bar(((testo)));
%      disp('Press any key to continue');
      
 %     kft = kf-1;
 %     for i=1:kft,
 %       h(i)=sum(dS(i:kft).^2);
 %       hh(i)=syh*kft*(kft-i+1)*((log(log(N)))^(1+0.01))/N+dS(kft)^2;  
 %     end;      
%      ndef = sum(h>hh);   
      title('Select model order in command window.')
      disp(['default=', num2str(ndef)])
%      figure
%      plot(h);
%      hold on
%      plot(hh,':');
      n=input('Select model order:(''Return'' gives default) ');
      if isempty(n)
         %testo=log(diag(SS));
         n=ndef;
         disp(['Order chosen to ',int2str(n)]);
     end
      close;
%      close;
      else n=nd; 
end  
 
S1=S(1:n,1:n);
Gammak= U(:,1:n)*S1^(1/2);

Gammak1=Gammak(1:kf*p-p,:);


if strcmp(Aest,'TLS') % total least squares solution 
   U1=Gammak1;
   U2=Gammak(p+1:p*kf,:);
   [U S V]=svd([U1 U2]);
   V12=V(1:n,n+1:2*n);
   V22=V(n+1:2*n,n+1:2*n); 
   Ad=-V12*pinv(V22);
   Cd=Gammak(1:p,:);

elseif strcmp(Aest,'SI')
   U1=Gammak1;
   U2=Gammak(p+1:p*kf,:);
   Ad = pinv(U1)*U2;
   Cd=Gammak(1:p,:);

elseif strcmp(Aest,'ls')
  pGammak=pinv(Gammak); 
  if strcmp(type,'cva')
   pGammak = pGammak*pinv(Lup');
end;
if  strcmp(BD,'ls')

   [UUU1,SSS1,VVV1]=svd(R(1:m*kf,1:m*(kp+kf)));
   [rr cc]=size(SSS1);
   
   UUU1 = UUU1(:,1:rr);
   SSS1=SSS1(1:rr,1:rr);
   VVV1=VVV1(:,1:rr);

  Zk1 = [pGammak*R((kf+kp)*m+p*kp+p+1:(kf+kp)*m+(kf+kp)*p,1:(kf+kp)*m); R((kp+kf)*m+kp*p+1:(kp+kf)*m+kp*p+p,1:(kp+kf)*m)];
  Zk = [pGammak*R((kf+kp)*m+p*kp+1:(kf+kp)*m+(kf+kp)*p-p,1:(kf+kp)*m); VVV1'];
  pZk=pinv(Zk);
  Theta = Zk1*pZk;
  Ad = Theta(1:n,1:n);
  Cd = Theta(n+1:n+p,1:n);
  
  else


  Zk1x = [pGammak*R((kf+kp)*m+p*kp+p+1:(kf+kp)*m+(kf+kp)*p,kf*m+1:(kf+kp)*m)];
  Zkx = [pGammak*R((kf+kp)*m+p*kp+1:(kf+kp)*m+(kf+kp)*p-p,kf*m+1:(kf+kp)*m)];
  pZkx=pinv(Zkx);
  Ad = Zk1x*pZkx;
  Cd = R((kf+kp)*m+p*kp+1:(kf+kp)*m+p*kp+p,kf*m+1:(kf+kp)*m)*pZkx;
end;
%Gammak = [Lup'*Gammak; Cd*Ad^(kd-1)];
else disp('Wrong choice of estimatin method for A');    
end;  

 

% filter out initial condition
Gammaext=Cd;
 for i=1:(kf+kp-1),
     Gammaext = [Cd ; Gammaext*Ad];
  end;
  
 Gammak=Gammaext(1:kbd*p,:);
  
  z = sqrt(N)*filter_initial_gamma(Ys,Gammaext,p);

  [As,Ks,Cs,Lambda]=stoc_est_cva(z,[ks ks],ns);


[ns ns1] = size(As);
[nd nd1] = size(Ad);

At = [Ad zeros(nd,ns) ; zeros(ns,nd) As];
Ct = [Cd Cs];

Gammas=Cs;
 for i=1:(kbd-1),
     Gammas = [Cs ; Gammas*As];
  end;
  

  Ds = chol(Lambda)';
  Bs = Ks*Ds;
Hs = zeros(kbd*p,kbd*p);
for i=1:kbd;
   Hs((i-1)*p+1:i*p,(i-1)*p+1:i*p)= Ds;
   Hs(i*p+1:kbd*p,(i-1)*p+1:i*p)=Gammas(1:(kbd-i)*p,:)*Bs;
end;

Gammat=Ct;
 for i=1:(kbd-1),
     Gammat = [Ct ; Gammat*At];
  end;
  [Ugd Sgd Vgd] = svd([Gammak]);
  [Ugs Sgs Vgs] = svd([Gammas]);
  [rug cug]=size(Ugd);
  Ugdp = Ugd(:,nd+1:cug);
  Ugs = Ugs(:,1:ns);
  [Ug Sg Vg] = svd(Ugs*Ugs'*Ugdp*Ugdp');
  na = sum(diag(Sg) > 0);
  
  Gammat = [Gammak Ug(:,1:na)*Sg(1:na,1:na)];
  Gammat = [Gammak Gammas];
if strcmp(BD,'ls')

  
    
  [Bd,Dd]=BD_ls_new(Ad,Cd,[pGammak zeros(n,p)],Theta,diag(SSS1),UUU1,m,delay);


elseif strcmp(BD,'msm_block')
   Ncol = floor(T/((kf+kp)));
   Yd = Yd(:,1:Ncol); 
   [Bd,Dd,x0]=BD_msm_block(Ad,Cd,yd,u,(kf+kp),Ncol,delay);
elseif strcmp(BD,'msm')
   [Bd,Dd,x0]=BD_msm(Ad,Cd,y,u,T,delay);
elseif strcmp(BD,'mpm')
   [Bd,Dd,x0]=BD_mpm(Ad,Cd,As,Cs,Ks,y,u,T,delay); 
elseif strcmp(BD,'proj_ls')
   Gammad = [Gammak];
   [Bd,Dd]=BD_proj_ls(Gammad,Phi,UUU,SSS,kbd,delay); 
elseif strcmp(BD,'proj_ls_white')
   
   Gammad =[Gammak];
   
   cov = covariance(u,2*kbd-1);
   LAMBDAu=[];
   
  for i=1:2*kbd
    for k=0:2*kbd-1
      if i-k <=0
        LAMBDAu(1+(i-1)*m:i*m,1+k*m:(k+1)*m)=cov(:,:,abs(k-i)+2)';
      else  
        LAMBDAu(1+(i-1)*m:i*m,1+k*m:(k+1)*m)=cov(:,:,i-k);
      end;
     end;
  end;
  
  [Bd,Dd]=BD_proj_ls_white_7_00(Gammat,Gammad,Phi,UUU,SSS,N,Hs,LAMBDAu,kbd,delay);  
 elseif strcmp(BD,'proj_ls_white_new')
   Gammad =[Gammak];
   
   cov = covariance(u,2*kbd-1);
   LAMBDAu=[];
   
  for i=1:2*kbd
    for k=0:2*kbd-1
      if i-k <=0
        LAMBDAu(1+(i-1)*m:i*m,1+k*m:(k+1)*m)=cov(:,:,abs(k-i)+2)';
      else  
        LAMBDAu(1+(i-1)*m:i*m,1+k*m:(k+1)*m)=cov(:,:,i-k);
      end;
     end;
  end;
 
    [Bd,Dd]=BD_proj_ls_white_12_00(Gammat,Gammad,Phi,UUU,SSS,N,Hs,LAMBDAu,kbd,delay);  

 
elseif strcmp(BD,'proj_ls_white_d')
   Gammad = [Gammak];
      cov = covariance(u,2*kbd-1);
   LAMBDAu=[];
   
  for i=1:2*kbd
    for k=0:2*kbd-1
      if i-k <=0
        LAMBDAu(1+(i-1)*m:i*m,1+k*m:(k+1)*m)=cov(:,:,abs(k-i)+2)';
      else  
        LAMBDAu(1+(i-1)*m:i*m,1+k*m:(k+1)*m)=cov(:,:,i-k);
      end;
     end;
  end;
  [Bd,Dd]=BD_proj_ls_whited_7_00(Gammad,Phi,UUU,SSS,N,Hs,LAMBDAu,kbd,delay);  

elseif strcmp(BD,'proj')
   Gammad = [Gammak];

   [Bd,Dd]=BD_proj(Gammad,Phi,kf,m,delay); 

else disp('Wrong choice of method for estimation of B and D');
   return
end;

