% function [A,B,C,D,K,Lambda]=subspace_id(y,u,nn,nd,k,BD,T,delay,type,Aest);
%
%
% (A,B,C,D,K,I,Lambda) is an innovation representation of the system
%  Lambda is the variance of the innovation
%
%  y : output data as row vector y = [y(1), y(2),......,y(Ndat)]
%  u : input data as row vector u = [u(1), u(2),......,u(Ndat)]
%  k : dimension of spaces 
%  nn:  is the choosen order, if entered as row vector (e.g. nn = [1: 10])
%       displays information about those orders
%  nd: order of deterministic subsystem
%  type : identification of the deterministic part ;
%      type = 'cva' : Canonical Variate Analysis
%      type = 'cva_stoc' : Canonical Variate Analysis on residuals
%      type = 'ort' : orthogonal projection
%      type = 'obl' : oblique projection
%  BD : identification of the matrices B and D                      % DO  NOT USE 
%      BD = 'cva' use cva approach with approximated states         % DO  NOT USE
%      BD = 'cva_new' use ''my'' cva approach with exact states,    % DO  NOT USE
%           get only approximated B and D                           % DO  NOT USE
%  BD = 'cva_new2' use ''my'' cva approach with exact states,       % DO  NOT USE
%           get  B and D with "oblique" states .(approximated)      % DO  NOT USE
%      BD = 'proj' is the projection approach (e.g. MOESP)
%      BD = 'proj_ls' is the projection approach with 
%           orthonormalization
%      BD = 'proj_ls_red' is the projection approach with 
%           orthonormalization and reduction of observability space
%      BD = 'proj_ls_red_alt' is the projection approach with          % DO  NOT USE
%           orthonormalization and reduction of observability space    % DO  NOT USE
%           usign directly deterministic observability matrix and      % DO  NOT USE
%           changing basis after having found B,D.                     % DO  NOT USE
%
%      BD = 'proj_ls_white' is the projection approach with pre-withening of the residuals
%      BD = 'mpm'  is the minimum prediction error method
%      BD = 'ls'   is the Least squares estimate obtained with the orthonormalization
%                  of future inputs
%      BD = 'lsr'   is the robust algorithm of VODM
%      BD = 'lsvo'  is the original n4sid
%      BD = 'msm_block' : minimum simulation error on the columns of the 
%                         estimated deterministic component
%
% default : BD = 'proj_ls_white';
%           type = 'cva';
% Aest : choice of identification method for A
%        Aest = 'TLS' total least square solution
%        Aest = 'SI' shift invariance
%        Aest = 'ls' with states
% need to use Aest ='ls' with BD ='ls'
% default Aest ='ls'
%  
%
% note that BD='ls' with type = 'ort' gives the same estimate of (A,C) than type='ort' and Aest='SI';

function [A,B,C,D,K,Lambda,T]=subspace_id(y,u,nn,nd,k,BD,T,delay,type,Aest);

if ((nargin > 10) | (nargin < 2)) 
   disp('Wrong number of input arguments')
   return
end;

[m,Nu]=size(u);
[p,Ny]=size(y);


if ((nargin < 3) | isempty(nn))
   nn=[1:10];
end;

if ((nargin < 4) | isempty(nd))
   nd=[1:10];
end;

if ((nargin < 5) | isempty(k))
   k = ceil(1.5*max(nn)+3); 
end;

if (nargin < 6  | isempty(BD))
   BD = 'ls';
end;

if (nargin < 7 | isempty(T)) 
   T = 100;
end;

if (nargin < 8  | isempty(delay))
   delay = zeros(1,m);
end;

if (nargin < 9  | isempty(type))
   type = 'ort';
end;

if (nargin < 10  | isempty(Aest))
   Aest = 'SI';
end;


N = min(Nu,Ny)-2*k+1;

%%% Hankel mastrices:

U = Hankel(u,2*k,N);
Y = Hankel(y,2*k,N);
Up = U(1:k*m,:);


if (strcmp(BD,'cva_new')|strcmp(BD,'cva_new2'))
for i =1:k,
   Uf((i-1)*m+1:i*m,:) = U((2*k-i)*m+1:(2*k-i+1)*m,:);
end;
else
Uf = U(k*m+1:2*k*m,:);
end;

Yp = Y(1:k*p,:);
Yf = Y(k*p+1:2*k*p,:);

H = [Uf; Up ; Y];

[Q,R] =qr(H',0); % economy size qr decomposition 

Q=Q(:,1:2*k*(m+p))';
R=R(1:2*k*(m+p),1:2*k*(m+p))';

Qyd = Q(1:2*k*m,:);
kf=k;
kp=k;
Qys = Q((kf+kp)*m+1:(kf+kp)*(m+p),:);

Yd = R(2*k*m+1:2*k*(m+p),1:2*k*m)*Qyd;
Ys = R((kf+kp)*m+1:(kf+kp)*(m+p),(kf+kp)*m+1:(kf+kp)*(m+p))*Qys; %Y -Yd;
W=Ys(kp*p+1:(kf+kp)*(p),:);

if (strcmp(BD,'proj') | strcmp(type,'obl')),
 M = R(1:k*(2*m+p),1:k*(2*m+p));
 L = R(k*(2*m+p)+1:2*k*(p+m)-p,1:2*k*m+k*p)/M;
 Phi = L(:,1:k*m);
 LL = [zeros(k*p-p,k*m) L(:,k*m+1:2*k*m+k*p)]*M;
end;

if (strcmp(BD,'proj_ls') | strcmp(BD,'proj_ls_red')| strcmp(BD,'proj_ls_red_alt')|strcmp(BD,'proj_ls_white'))
   [UUU SSS VVV] = svd(R(1:k*m-m,1:k*m-m));
    rr = sum(diag(SSS)>eps);
   UUU = UUU(:,1:rr);
   SSS = SSS(1:rr,1:rr);
   VVV = VVV(:,1:rr);
   Phi = R(2*k*m+k*p+1:2*k*(m+p)-p,1:k*m-m)*VVV;
 end;
 

% compute matrices A,C
if (strcmp(Aest,'ls') & ~(strcmp(BD,'lsr')|strcmp(BD,'lsvo')))
if strcmp(type,'ort')
   [U S V] = svd(R(k*(2*m+p)+1:2*k*(p+m)-p,k*m+1:2*k*m+k*p)); % orthogonal projection
   Lup = eye((k-1)*p);   
elseif strcmp(type,'cva_stoc')
   Yup = R(2*k*m+k*p+1:2*k*(m+p)-p,2*k*m+k*p+1:2*k*(m+p)-p);
   Lup = chol(Yup*Yup');
    [U S V] = svd(pinv(Lup')*R(k*(2*m+p)+1:2*k*(p+m)-p,k*m+1:2*k*m+k*p)); % cva
   [ru cu]=size(U);
   S=S(1:cu,1:cu);
   UL = Lup'*U;
 elseif strcmp(type,'cva')
    Yup = R(2*k*m+k*p+1:2*k*(m+p)-p,k*m+1:2*k*(m+p)-p);
    Lup = chol(Yup*Yup');
    [U S V] = svd(pinv(Lup')*R(k*(2*m+p)+1:2*k*(p+m)-p,k*m+1:2*k*m+k*p)); % cva
   [ru cu]=size(U);
   S=S(1:cu,1:cu);
   UL = Lup'*U;

elseif strcmp(type,'obl')
   [U S V] = svd(LL);
   [UU SS VV] = svd(R(k*(2*m+p)+1:2*k*(p+m)-p,k*m+1:2*k*m+k*p)); 
   Lup = eye((k-1)*p);
   % oblique projection
else
   disp('Wrong choice of estimation method for Gamma')
   return
end;

else
   if strcmp(type,'ort')
   [U S V] = svd(R(k*(2*m+p)+1:2*k*(p+m),k*m+1:2*k*m+k*p)); % orthogonal projection
   Lup = eye(k*p);   
elseif strcmp(type,'cva_stoc')
   Yup = R(2*k*m+k*p+1:2*k*(m+p),2*k*m+k*p+1:2*k*(m+p));
   Lup = chol(Yup*Yup');
    [U S V] = svd(pinv(Lup')*R(k*(2*m+p)+1:2*k*(p+m),k*m+1:2*k*m+k*p)); % cva
   [ru cu]=size(U);
   S=S(1:cu,1:cu);
 elseif strcmp(type,'cva')
    Yup = R(2*k*m+k*p+1:2*k*(m+p),k*m+1:2*k*(m+p));
    Lup = chol(Yup*Yup');
    [U S V] = svd(pinv(Lup')*R(k*(2*m+p)+1:2*k*(p+m),k*m+1:2*k*m+k*p)); % cva
   [ru cu]=size(U);
   S=S(1:cu,1:cu);
  
elseif strcmp(type,'obl')
   M = R(1:k*(2*m+p),1:k*(2*m+p));
   L = R(k*(2*m+p)+1:2*k*(p+m),1:2*k*m+k*p)/M;
   Phi = L(:,1:k*m);
   LL = [zeros(k*p,k*m) L(:,k*m+1:2*k*m+k*p)]*M;
   [U S V] = svd(LL);
   Lup = eye(k*p);
else
   disp('Wrong choice of estimation method for Gamma')
   return
end;
end; %strcmp(Aest,'ls')



 if (strcmp(BD,'proj_ls_red') | strcmp(BD,'proj_ls_red_alt'))
    %compute deterministic observability matrix
    [Ud Sd Vd] = svd(R(2*k*m+k*p+1:2*k*(m+p)-p,m*k+1:2*m*k));
            
    nnd=nd;
    if min(nnd)<max(nnd),     
      figure
      dSd=diag(Sd);
      testod=(diag(Sd));
    
      ndefd=max(max(find(testod>(max(testod)+min(testod))/2)));
      bar(((testod)));
      title('Select model order in command window.')
      disp(['default=', num2str(ndefd)])
      
      nd=input('Select model order:(''Return'' gives default) ');
      if isempty(nd)
         %testo=log(diag(SS))
         nd=ndefd;
         disp(['Order chosen to ',int2str(nd)]);
      end
      close;
      else nd=nnd;
      end   
         Gamma_d = Ud(:,1:nd);
      
 end;


if min(nn)<max(nn),     
      figure
      dS=diag(S);
      testo=(diag(S));
      ndef=max(max(find(testo>(max(testo)+min(testo))/2)));
      bar(((testo)));
      title('Select model order in command window.')
      disp(['default=', num2str(ndef)])
      
      n=input('Select model order:(''Return'' gives default) ');
      if isempty(n)
         n=ndef;
         disp(['Order chosen to ',int2str(n)]);
      end
      close;
      else n=nn;
end  
 
S1=S(1:n,1:n);
U1=U(:,1:n);
V1=V(1:n,:);
Gammak= U(:,1:n)*S1^(1/2);



if ~strcmp(Aest,'ls')
 if (strcmp(type,'cva') | strcmp(type,'cva_stoc'))
    Gammakw = Lup'*Gammak;  
 else
    Gammakw=Gammak;
 end;   
 Gammak1=Gammakw(1:k*p-p,:);
 U1=Gammak1;
 U2=Gammakw(p+1:p*k,:);
 if strcmp(Aest,'TLS') % total least squares solution 
    [UU SS VV]=svd([U1 U2]);
    V12=VV(1:n,n+1:2*n);
    V22=VV(n+1:2*n,n+1:2*n);
    A=-V12*pinv(V22);
 elseif strcmp(Aest,'SI')
    A = pinv(U1)*U2; 
 else disp('Wrong choice of estimation method for A');   
 end;   
 if strcmp(BD,'proj_ls_red')
 Gamma_d1=Gamma_d(1:k*p-p,:);
 Gamma_d2=Gamma_d(p+1:k*p,:);
 Ad = pinv(Gamma_d1)*Gamma_d2;
 
 
 end;
 C=Gammak(1:p,:); 
end;

%%%%%%%%%%%%%%%%%%%%%%
%Compute residuals for calculation of steady state Kalman gain
%%%%%%%%%%%%%%%%%%%%%%



if ~(strcmp(Aest,'ls')& (strcmp(BD,'lsr')|strcmp(BD,'lsvo')))

pGammak=pinv(Gammak(1:p*(k-1),:));

if (strcmp(type,'cva')|strcmp(type,'cva_stoc'))
   if strcmp(Aest,'ls')      
      pGammak = pGammak*pinv(Lup');
      %pGammak = pinv(Lup'*Gammak);
   else
      pGammak = pGammak*pinv(Lup(1:p*(k-1),1:p*(k-1))');
   end;
end;
%if strcmp(type,'cva_stoc')
%   pGammak = pinv(Lup'*Gammak);
%end;

pGammakp=pGammak;

Zk10 = [pGammakp*R(2*m*k+p*k+p+1:2*m*k+2*p*k,1:2*k*m+(k+1)*p);
   R(2*m*k+p*k+1:2*m*k+p*k+p,1:2*m*k+p*(k+1))];
   Zk1 = Zk10(:,1:2*k*m+k*p);
   Zk = [pGammakp*R(2*m*k+p*k+1:2*m*k+2*p*k-p,1:2*k*m+k*p);
   R(1:m*k,1:2*m*k+p*k)];

Zk1x = [pinv(Gammak(1:p*(k-1),:))*R(2*m*k+p*k+p+1:2*m*k+2*p*k,k*m+1:2*k*m+k*p)];
Zkx = [pinv(Gammak(1:p*(k-1),:))*R(2*m*k+p*k+1:2*m*k+2*p*k-p,k*m+1:2*k*m+k*p)];
AA = Zk1x*pinv(Zkx);

%***********************************************
% orthonormalize set of equations
if  strcmp(BD,'ls')

   [UUU1,SSS1,VVV1]=svd(R(1:m*k,1:2*m*k+p*k));
   [rr cc]=size(SSS1);
   
   UUU1 = UUU1(:,1:rr);
   SSS1=SSS1(1:rr,1:rr);
   VVV1=VVV1(:,1:rr);

   Zk = [Zk(1:n,:);
      VVV1'];
end;

Theta = Zk1/Zk;

%%% Extraction of system matrices:
if strcmp(Aest,'ls')
   
if ~(strcmp(BD,'cva')|strcmp(BD,'cva_new')|strcmp(BD,'cva_new2'))
   %strcmp(BD,'ls')
   A = Theta(1:n,1:n);   
   C = Theta(n+1:n+p,1:n);
elseif   strcmp(BD,'cva')
   % compute oblique projection
   M = R(1:k*(2*m+p),1:k*(2*m+p));
   L = R(k*(2*m+p)+1:2*k*(p+m)-p,1:2*k*m+k*p)/M;
   L = L(:,k*m+1:2*k*m+k*p);
   PSI = pGammakp*L;
   Xk = [PSI*[Up ; Yp];
      Uf(1:m,:)];
      
   
   Xk1 = [PSI*[Up(m+1:k*m,:) ; Uf(1:m,:) ; Yp(p+1:k*p,:) ;Yf(1:p,:)];
      Yf(1:p,:)];
  

   Thetacva = Xk1/Xk;
   A = Thetacva(1:n,1:n);
   C = Thetacva(n+1:n+p,1:n);
   B = Thetacva(1:n,n+1:n+m);
   D = Thetacva(n+1:n+p,n+1:n+m);
 elseif strcmp(BD,'cva_new')
      
     X1 = [pGammakp*R(2*m*k+p*k+p+1:2*m*k+2*p*k,(k-1)*m+1:2*k*m+(k+1)*p) ;
        R(2*m*k+p*k+1:2*m*k+p*k+p,(k-1)*m+1:2*m*k+p*(k+1))];
     Xk1AC = X1(:,m+1:k*m+k*p+m);
     Xk1BD = X1(:,1:m);
     
     XkAC = pGammakp*R(2*m*k+p*k+1:2*m*k+2*p*k-p,k*m+1:2*k*m+k*p) ;
     XkBD = R((k-1)*m+1:k*m,(k-1)*m+1:k*m);
     
     ThetaAC = Xk1AC/XkAC;
     ThetaBD = Xk1BD/XkBD;
        A = ThetaAC(1:n,1:n);
        C = ThetaAC(n+1:n+p,1:n);
        B = ThetaBD(1:n,1:m);
        D = ThetaBD(n+1:n+p,1:m);
 
        % compute estimate for E[xv]E[vv']^(-1)
        % need oblique predictor space
        
         % compute oblique projection
     M = R(1:k*(2*m+p),1:k*(2*m+p));
     L = R(k*(2*m+p)+1:2*k*(p+m)-p,1:2*k*m+k*p)/M;
     L = L(:,k*m+1:2*k*m+k*p);
     Xk= pGammakp*L*R(m*k+1:2*m*k+k*p,(k-1)*m+1:k*m);
   
     KBD = Xk*XkBD'*pinv(XkBD*XkBD');
     B = B-A*KBD;
     D = D-C*KBD;
  elseif strcmp(BD,'cva_new2')
       % compute oblique projection
       M = R(1:k*(2*m+p),1:k*(2*m+p));
       L = R(k*(2*m+p)+1:2*k*(p+m)-p,1:2*k*m+k*p)/M;
       L = L(:,k*m+1:2*k*m+k*p);
       Xk = [pGammakp*L*[Up ; Yp];
             Uf((k-1)*m+1:k*m,:) ;Uf(1:(k-1)*m,:)];
       M = R(1:k*(2*m+p),1:k*(2*m+p));
       L = R(k*(2*m+p)+p+1:2*k*(p+m),1:2*k*m+k*p)/M;
       L = L(:,(k-1)*m+1:2*k*m+k*p);
       Xk1 = [pGammakp*L*[Uf((k-1)*m+1:k*m,:) ; Up ; Yp];
             Yf(1:p,:)];
        Thetacva2 = Xk1*pinv(Xk);
       A = Thetacva2(1:n,1:n);
       C = Thetacva2(n+1:n+p,1:n);
       B = Thetacva2(1:n,n+1:n+m);
       D = Thetacva2(n+1:n+p,n+1:n+m);
       
     % refine estimate of A,C with exact formulas
          
     X1 = [pGammakp*R(2*m*k+p*k+p+1:2*m*k+2*p*k,(k-1)*m+1:2*k*m+(k+1)*p) ;
        R(2*m*k+p*k+1:2*m*k+p*k+p,(k-1)*m+1:2*m*k+p*(k+1))];
     Xk1AC = X1(:,m+1:k*m+k*p+m);
     
     XkAC = pGammakp*R(2*m*k+p*k+1:2*m*k+2*p*k-p,k*m+1:2*k*m+k*p) ;
     
     ThetaAC = Xk1AC/XkAC;
        Ac = ThetaAC(1:n,1:n);
        Cc = ThetaAC(n+1:n+p,1:n);
   
end;   

end; %if strcmp(Aest,'ls')



   WW = Zk1 - Theta*Zk;
   WWr=Zk10(:,2*k*m+k*p+1:2*k*m+(k+1)*p);
   QRS=WW*WW'+WWr*WWr';
   Q = QRS(1:n,1:n);
   S = QRS(1:n,n+1:n+p);
   R = QRS(n+1:n+p,n+1:n+p);
 
   
% solve Riccati equation 

 pR =pinv(R);
 F = A-S*pR*C;
 Qt = Q -S*pR*S';
 
 L=[F' zeros(n,n) ;-Qt eye(n)];
 M = [eye(n) C'*pR*C ; zeros(n,n) F];
 
 [Z DD] =eig(L,M);  % generalized eigenvalue eigenvector problem
 [lambda,ii] = sort(abs(diag(DD))); % sort eigenvalues inside the unit disc
 if (~((lambda(n) < 1) & (lambda(n+1)>1)))
           disp('Can''t order eigenvalues');
 end
 P = real(Z(n+1:2*n,ii(1:n))/Z(1:n,ii(1:n))); % state covariance
 
 Lambda = C*P*C'+R; % Variance of innovation
 K = (A*P*C'+S)*pinv(Lambda); % Kalman Gain
 
 %if (strcmp(type,'cva') | strcmp(type,'cva_stoc'))
 %   Gammakw = Lup'*Gammak;  
 %else
 %   Gammakw=Gammak;
 %end;   
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else
   
  
   Gammakw = Lup'*Gammak;
   pGammak1=pinv(Gammakw(1:p*(k-1),:));
   pGammak = pinv(Gammakw(1:p*k,:));
   


 Zk10 = [pGammak1*R(2*m*k+p*k+p+1:2*m*k+2*p*k,1:2*k*m+(k+1)*p);
   R(2*m*k+p*k+1:2*m*k+p*k+p,1:2*m*k+p*(k+1))];
   Zk1 = Zk10(:,1:2*k*m+k*p);
   Zk = [pGammak*R(2*m*k+p*k+1:2*m*k+2*p*k,1:2*k*m+k*p);
   R(1:m*k,1:2*m*k+p*k)];

   if (strcmp(BD,'lsr')|strcmp(BD,'ls')) % attenzione: cambiata il 11/1/04 da controllare bene ! 
   %if strcmp(BD,'lsr')
   [UUU1,SSS1,VVV1]=svd(R(1:m*k,1:2*m*k+p*k));
   [rr cc]=size(SSS1);
   
   UUU1 = UUU1(:,1:rr);
   SSS1=SSS1(1:rr,1:rr);
   VVV1=VVV1(:,1:rr);
   
   else
   VVV1 = R(1:m*k,1:2*m*k+p*k)';
   SSS1 = eye(m*k);
   UUU1 = eye(m*k);
   end;
   Zk = [Zk(1:n,:);
      VVV1'];

Theta = Zk1/Zk;

%%% Extraction of system matrices:
   A = Theta(1:n,1:n);   
   C = Theta(n+1:n+p,1:n);
   

   WW = Zk1 - Theta*Zk;
   WWr=Zk10(:,2*k*m+k*p+1:2*k*m+(k+1)*p);
   QRS=WW*WW'+WWr*WWr';
   Q = QRS(1:n,1:n);
   S = QRS(1:n,n+1:n+p);
   R = QRS(n+1:n+p,n+1:n+p);
 
   
% solve Riccati equation 
   
 pR =pinv(R);
 F = A-S*pR*C;
 Qt = Q -S*pR*S';
 
 L=[F' zeros(n,n) ;-Qt eye(n)];
 M = [eye(n) C'*pR*C ; zeros(n,n) F];
 
 [Z DD] =eig(L,M);  % generalized eigenvalue eigenvector problem
 [lambda,ii] = sort(abs(diag(DD))); % sort eigenvalues inside the unit disc
 if (~((lambda(n) < 1) & (lambda(n+1)>1)))
           disp('Can''t order eigenvalues');
 end
 P = real(Z(n+1:2*n,ii(1:n))/Z(1:n,ii(1:n))); % state covariance
 
 Lambda = C*P*C'+R; % Variance of innovation
 K = (A*P*C'+S)*pinv(Lambda); % Kalman Gain
 
end;

 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 Gammakw = C;
 for i=1:k-2;
    Gammakw = [C ; Gammakw*A];
 end;
 
 if strcmp(BD,'ls')

   
    
    [B,D]=BD_ls_new(A,C,[pGammak zeros(n,p)],Theta,diag(SSS1),UUU1,m,delay);
   
   elseif (strcmp(BD,'lsr')|strcmp(BD,'lsvo'))
  
    %Gammakw =Lup'*Gammak;  
   [B,D]=BD_ls(A,C,pGammak,pGammak1,Theta,diag(SSS1),UUU1,m,delay);

elseif strcmp(BD,'proj_ls_white')
   Gamma = [C];
   for i=1:k-2,
      Gamma = [C ; Gamma*A];
   end;
     Ds = chol(Lambda)';
     Bs = K*Ds;
    Hs = zeros(k*p-p,k*p-p);
  for i=1:k-1;
    Hs((i-1)*p+1:i*p,(i-1)*p+1:i*p)= Ds;
    Hs(i*p+1:(k-1)*p,(i-1)*p+1:i*p)=Gamma(1:(k-1-i)*p,:)*Bs;
  end;

   cov = covariance(u,2*kf-1);
   LAMBDAu=[];
  for i=1:2*kf
    for k=0:2*kf-1
      if i-k <=0
        LAMBDAu(1+(i-1)*m:i*m,1+k*m:(k+1)*m)=cov(:,:,abs(k-i)+2)';
      else  
        LAMBDAu(1+(i-1)*m:i*m,1+k*m:(k+1)*m)=cov(:,:,i-k);
      end;
     end;
  end;
  kbd=kf-1;
  [B,D]=BD_proj_ls_white_7_00(Gamma,Gamma,Phi,UUU,diag(SSS),N,Hs,LAMBDAu,kbd,delay); 
elseif strcmp(BD,'proj_ls')
  Gamma = [C];
   for i=1:kf-2,
      Gamma = [C ; Gamma*A];
   end;

   [B,D]=BD_proj_ls(Gamma,Phi,UUU,diag(SSS),kf-1,delay);
elseif strcmp(BD,'proj_ls_red')
    Gamma = [C];
   for i=1:k-2,
      Gamma = [C ; Gamma*A];
   end;
   [T DD]=eig(A);
   cond(T);
   [B,D]=BD_proj_ls_red_dec2(T,Gamma,Gamma_d,Phi,UUU,diag(SSS),k-1,delay);
elseif strcmp(BD,'proj_ls_red_alt')
  
   [B,D]=BD_proj_ls_red_alt(Gammakw,Gamma_d,Phi,UUU,diag(SSS),k-1,delay);
elseif strcmp(BD,'msm_block')
   Ncol = floor(T/(2*k));
   Yd = Yd(:,1:Ncol); 
   yd = sqrt(N)*Yd(:);
   [B,D,x0]=BD_msm_block(A,C,yd,u,2*k,Ncol,delay);
elseif strcmp(BD,'proj')
   Gamma = [C];
   for i=1:k-2,
      Gamma = [C ; Gamma*A];
   end;
   [B,D]=BD_proj(Gamma,Phi,k-1,m,delay);
elseif strcmp(BD,'mpm')
   [B,D,x0]=BD_mpm_joint(A,C,K,y,u,T,delay);
elseif strcmp(BD,'msm')
   [B,D,x0]=BD_msm(A,C,y,u,T,delay);
elseif ~(strcmp(BD,'cva')|strcmp(BD,'cva_new')|strcmp(BD,'cva_new2'))
   disp('Wrong choice of estimation method for (B,D)')
   return
end;


